
#include "stdafx.h"
#include "SubControls.h"

void AKScreenToClient(HWND m_phWnd, LPRECT lpRect);

//////////////////////////////////////////////////
// CMemDC - memory DC
//
// Author: Keith Rule
// Email:  keithr@europa.com
// Copyright 1996-1997, Keith Rule
//
// You may freely use or modify this code provided this
// Copyright is included in all derived versions.
//
// This class implements a memory Device Context

class CMemDC : public CDC {
private:
	CBitmap*	m_bitmap;
	CBitmap*	m_oldBitmap;
	CDC*		m_pDC;
	CRect		m_rcBounds;
public:
	CMemDC(CDC* pDC, const CRect& rcBounds) : CDC()
		{
		CreateCompatibleDC(pDC);
		m_bitmap = new CBitmap;
		m_bitmap->CreateCompatibleBitmap(pDC, rcBounds.Width(), rcBounds.Height());
		m_oldBitmap = SelectObject(m_bitmap);
		m_pDC = pDC;
		m_rcBounds = rcBounds;
		}
	~CMemDC() 
		{
		m_pDC->BitBlt(m_rcBounds.left, m_rcBounds.top, m_rcBounds.Width(), m_rcBounds.Height(), 
					this, m_rcBounds.left, m_rcBounds.top, SRCCOPY);
		SelectObject(m_oldBitmap);
		if (m_bitmap != NULL) delete m_bitmap;
		}
	CMemDC* operator->() 
		{
		return this;
		}
};

/**********************************************************************/

IMPLEMENT_DYNAMIC(CListBoxD, CListBox)

/////////////////////////////////////////////////////////////////////////////
// CListBoxD

CListBoxD::CListBoxD()
{
}

CListBoxD::~CListBoxD()
{
	DestroyWindow();
}


BEGIN_MESSAGE_MAP(CListBoxD, CListBox)
	//{{AFX_MSG_MAP(CListBoxD)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CListBoxD, CListBox)
	//{{AFX_DISPATCH_MAP(CListBoxD)
		// HINWEIS - Der Klassen-Assistent fgt hier Zuordnungsmakros ein und entfernt diese.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CListBoxD 

BOOL CListBoxD::OnEraseBkgnd(CDC* pDC) 
{
	UNUSED_ALWAYS(pDC);
	return false;	
}

void CListBoxD::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect winRect,itemRect;

	GetClientRect(&winRect);
	GetItemRect(0,itemRect);
/*
	Nicht sehr schn, aber das ist schlielich nur eine Demo!
*/
	winRect.right = itemRect.right;
	
	CMemDC memDC(&dc,winRect);
	memDC.FillSolidRect(winRect,GetSysColor(COLOR_WINDOW));
	DefWindowProc(WM_PAINT,(WPARAM)memDC->m_hDC,(LPARAM)0);
}

/**********************************************************************/

void CListBoxD::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{

	CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);

	if (((LONG)(lpDrawItemStruct->itemID) >= 0) &&
		(lpDrawItemStruct->itemAction & (ODA_DRAWENTIRE | ODA_SELECT)))
		{
		int cyItem;
		cyItem = GetItemHeight(lpDrawItemStruct->itemID);
		BOOL fDisabled = false;
		itemRect = lpDrawItemStruct->rcItem;
		COLORREF newTextColor = fDisabled ?
			RGB(0x80, 0x80, 0x80) : GetSysColor(COLOR_WINDOWTEXT);  // light gray
		COLORREF oldTextColor = pDC->SetTextColor(newTextColor);

		COLORREF newBkColor = GetSysColor(COLOR_WINDOW);
		COLORREF oldBkColor = pDC->SetBkColor(newBkColor);


		if (!fDisabled && ((lpDrawItemStruct->itemState & ODS_SELECTED) != 0))
			{
			pDC->SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
			pDC->SetBkColor(GetSysColor(COLOR_HIGHLIGHT));
			}


		CString strText;
		GetText(lpDrawItemStruct->itemID, strText);

		pDC->ExtTextOut(lpDrawItemStruct->rcItem.left,
			lpDrawItemStruct->rcItem.top,
			ETO_OPAQUE, &(lpDrawItemStruct->rcItem), strText, strText.GetLength(), NULL);

		pDC->SetTextColor(oldTextColor);
		pDC->SetBkColor(oldBkColor);
		}

	if ((lpDrawItemStruct->itemAction & ODA_FOCUS) != 0)
		pDC->DrawFocusRect(&(lpDrawItemStruct->rcItem));
	
}

/**********************************************************************/

void CListBoxD::MeasureItem(LPMEASUREITEMSTRUCT ) 
{
	// sonst assert	
}

int CListBoxD::CompareItem(LPCOMPAREITEMSTRUCT ) 
{
	return 0;
}

/**********************************************************************/

CTabCtrlD::CTabCtrlD()
{
}

CTabCtrlD::~CTabCtrlD()
{
	DestroyWindow();
}


BEGIN_MESSAGE_MAP(CTabCtrlD, CTabCtrl)
	//{{AFX_MSG_MAP(CTabCtrlD)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_NOTIFY_REFLECT(TCN_SELCHANGING, OnSelchanging)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CTabCtrlD 


BOOL CTabCtrlD::OnEraseBkgnd(CDC* pDC) 
{	
	UNUSED_ALWAYS(pDC);
	return false;
}

/**********************************************************************/

void CTabCtrlD::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect rcWnd,rcChild;
	DWORD wStyle;

	GetWindowRect(&rcWnd);
	AKScreenToClient(GetSafeHwnd(),&rcWnd);

	CMemDC memDC(&dc,rcWnd);


	CWnd* pWndChild = GetParent()->GetWindow( GW_CHILD );

	while( pWndChild ) 
		{
		pWndChild->GetWindowRect(rcChild);
		AKScreenToClient(GetSafeHwnd(),rcChild );
		
		wStyle = pWndChild->GetStyle();

		// Testet ob Control vollstndig im TabCtrl ist
		if(pWndChild != this && wStyle & WS_VISIBLE)
			if(rcWnd.PtInRect(CPoint(rcChild.left,rcChild.top)) &&
				rcWnd.PtInRect(CPoint(rcChild.right,rcChild.bottom)))
				{
				dc.ExcludeClipRect(&rcChild);
				}

		pWndChild = pWndChild->GetNextWindow();
		}


	memDC.FillSolidRect(rcWnd,GetSysColor(COLOR_3DFACE));
	DefWindowProc(WM_PAINT,(WPARAM)memDC->m_hDC,(LPARAM)0);
}

/**********************************************************************/

void CTabCtrlD::OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CRect rcWnd,rcChild;

	UNUSED_ALWAYS(pNMHDR);

	GetWindowRect(&rcWnd);
	AKScreenToClient(GetSafeHwnd(),&rcWnd);

	CWnd* pWndChild = GetParent()->GetWindow( GW_CHILD );

	while( pWndChild ) 
		{
		pWndChild->GetWindowRect(rcChild);
		AKScreenToClient(GetParent()->GetSafeHwnd(),rcChild );

		// Testet ob Control vollstndig im TabCtrl ist
		if(pWndChild != this)
			if(rcWnd.PtInRect(CPoint(rcChild.left,rcChild.top)) &&
				rcWnd.PtInRect(CPoint(rcChild.right,rcChild.bottom)))
				pWndChild->ShowWindow(false);

		pWndChild = pWndChild->GetNextWindow();
		}

	*pResult = 0;
}

/**********************************************************************/

/////////////////////////////////////////////////////////////////////////////
// CListCtrlD

CListCtrlD::CListCtrlD()
{
	bFirst = true;
}

CListCtrlD::~CListCtrlD()
{
	DestroyWindow();
}


BEGIN_MESSAGE_MAP(CListCtrlD, CListCtrl)
	//{{AFX_MSG_MAP(CListCtrlD)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CListCtrlD 

BOOL CListCtrlD::OnEraseBkgnd(CDC* pDC) 
{
	UNUSED_ALWAYS(pDC);
	return false;
}

/**********************************************************************/

void CListCtrlD::OnPaint() 
{
	CRect clip,winRect,headerRect,itemRect;
	
	if(bFirst)
		ListView_SetExtendedListViewStyle(GetSafeHwnd(),
		LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_CHECKBOXES);
	bFirst = false;

	CPaintDC dc(this);
	GetWindowRect(&winRect);
	AKScreenToClient(GetSafeHwnd(),&winRect);

	GetDlgItem(0) -> GetWindowRect(&headerRect);
	AKScreenToClient(GetSafeHwnd(),&headerRect);

	dc.ExcludeClipRect(&headerRect);
	CMemDC memDC(&dc,winRect);

	memDC.GetClipBox(&clip);
	memDC.FillSolidRect(clip,GetSysColor(COLOR_WINDOW));


	DefWindowProc(WM_PAINT,(WPARAM)memDC->m_hDC,(LPARAM)0);
}

/**********************************************************************/
